from django.db import models


class Student(models.Model):
    sname = models.CharField(max_length=20, null=False)    
    def __str__(self):
        return '%s (id: %d)' % (self.sname, self.id)


class Teacher(models.Model):
    tname = models.CharField(max_length=20)
    def __str__(self):
        return '%s (id: %d)' % (self.tname, self.id)


class Course(models.Model):
    id = models.CharField(max_length=7, primary_key=True)
    cname = models.CharField(max_length=20)
    # Course <-> many-to-many <-> Teacher
    teachers = models.ManyToManyField(Teacher)
    def __str__(self):
        return '%s: %s' % (self.id, self.cname)


# Enrollment = Student <-> many-to-many <-> Course (with grade)
class Enrollment(models.Model):
    class Meta:
        unique_together = ('student', 'course')
    # Enrollment <-> many-to-one <-> Student
    student = models.ForeignKey(Student, on_delete=models.CASCADE)
    # Enrollment <-> many-to-one <-> Course
    course = models.ForeignKey(Course, on_delete=models.CASCADE)
    grade = models.CharField(max_length=1, null=True, blank=True)
    def __str__(self):
        return '%s, %s, %s' % (self.student, self.course, self.grade)
