from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import UserCreationForm
from django.db import connection
from django.http import Http404, HttpResponseServerError, HttpResponseBadRequest
from django.shortcuts import render, redirect

from myapp.models import *


@login_required
def profile(request):
    return render(request, 'profile.html')


def signup(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            form.save()
            print('User created')
            return redirect('login')
        else:
            print('Validation failed')
    else:
        form = UserCreationForm()
    context = {'form': form}
    return render(request, 'signup.html', context)


@login_required
def students(request):
    students = Student.objects.all().order_by('sname')
    context = {'students': students}
    return render(request, 'students.html', context)


@login_required
def student(request, sid):
    students = Student.objects.filter(id=sid)

    if not students.exists():
        print('Student %d not found' % sid)
        raise Http404('Student not found')
    student = students.first()

    if request.method == 'POST':  # add course
        cid = request.POST.get('cid')
        courses = Course.objects.filter(id=cid)
        if not courses.exists():
            print('Course %s not found' % cid)
            raise Http404('Course not found')
        course = courses.first()
        enrollments = Enrollment.objects.filter(student=sid, course=cid)
        if enrollments.exists():
            print('Student %d already enrolled in course %s' % (sid, cid))
        else:
            e = Enrollment()
            e.student = student
            e.course = course
            e.save()

    enrollments = student.enrollment_set.all().order_by('course__cname')
    print('%s' % enrollments)
    context = {'student': student, 'courses': enrollments}
    return render(request, 'student.html', context)


@login_required
def add(request, sid, cid):
    enrollments = Enrollment.objects.filter(student=sid, course=cid)
    if enrollments.exists():
        print('Student %d already enrolled in course %s' % (sid, cid))
    else:
        e = Enrollment()
        e.student = Student.objects.get(id=sid)
        e.course = Course.objects.get(id=cid)
        e.save()
    return redirect('student', sid=sid)


@login_required
def drop(request, sid, cid):
    try:
        e = Enrollment.objects.get(student=sid, course=cid)
        e.delete()
    except Exception as ex:
        print('Could not process enrollment: %s' % ex)
        return HttpResponseBadRequest('Could not process enrollment')
    return redirect('student', sid=sid)


@login_required
def grade(request, sid, cid, grade):
    if grade not in [1, 2]:  # 1=Pass, 2=Fail
        return HttpResponseBadRequest('Invalid grade')
    g = 'P' if grade == 1 else 'F'
    try:
        e = Enrollment.objects.get(student=sid, course=cid)
        e.grade = g
        e.save()
    except Exception as ex:
        print('Could not process enrollment: %s' % ex)
        return HttpResponseBadRequest('Could not process enrollment')
    return redirect('student', sid=sid)


@login_required
def courses(request):
    courses = Course.objects.all().order_by('cname')
    context = {'courses': courses}
    return render(request, 'courses.html', context)


@login_required
def course(request, cid):
    courses = Course.objects.filter(id=cid)
    if not courses.exists():
        print('Course %s not found' % cid)
        raise Http404('Course not found')
    course = courses.first()
    teachers = course.teachers.all()
    enrollments = course.enrollment_set.all().order_by('student__sname')
    context = {'course': course, 'teachers': teachers, 'students': enrollments}
    print('%s' % context)
    return render(request, 'course.html', context)
